# This file was auto-generated by Fern from our API Definition.

import typing

from .date_definition_with_includes import DateDefinitionWithIncludes
from .long_text_definition_with_includes import LongTextDefinitionWithIncludes
from .number_definition_with_includes import NumberDefinitionWithIncludes
from .rating_definition_with_includes import RatingDefinitionWithIncludes
from .select_definition_with_includes import SelectDefinitionWithIncludes
from .select_multi_definition_with_includes import SelectMultiDefinitionWithIncludes
from .text_definition_with_includes import TextDefinitionWithIncludes
from .toggle_definition_with_includes import ToggleDefinitionWithIncludes
from .user_multi_definition_with_includes import UserMultiDefinitionWithIncludes
from .user_single_definition_with_includes import UserSingleDefinitionWithIncludes

FieldDefinitionWithIncludes = typing.Union[
    DateDefinitionWithIncludes,
    LongTextDefinitionWithIncludes,
    NumberDefinitionWithIncludes,
    RatingDefinitionWithIncludes,
    SelectDefinitionWithIncludes,
    SelectMultiDefinitionWithIncludes,
    TextDefinitionWithIncludes,
    ToggleDefinitionWithIncludes,
    UserMultiDefinitionWithIncludes,
    UserSingleDefinitionWithIncludes,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel
from .user_multi_definition_field_configuration_custom_members_item_type import (
    UserMultiDefinitionFieldConfigurationCustomMembersItemType,
)


class UserMultiDefinitionFieldConfigurationCustomMembersItem(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    User Id
    """

    type: UserMultiDefinitionFieldConfigurationCustomMembersItemType

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel


class Action(UniversalBaseModel):
    active: bool = pydantic.Field()
    """
    Active Status
    """

    created_at: dt.datetime
    creator_id: str = pydantic.Field()
    """
    Creator ID
    """

    deleted_at: typing.Optional[dt.datetime] = None
    description: str = pydantic.Field()
    """
    Action Description
    """

    event: str = pydantic.Field()
    """
    Event Name
    """

    id: str = pydantic.Field()
    """
    Action ID
    """

    multi_asset: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Manages action trigger on multiple assets
    """

    name: str = pydantic.Field()
    """
    Action Name
    """

    timeout: typing.Optional[int] = pydantic.Field(default=None)
    """
    Timeout for the action requests in seconds
    """

    updated_at: dt.datetime
    url: str = pydantic.Field()
    """
    Action URL
    """

    workspace_id: str = pydantic.Field()
    """
    Workspace ID
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel
from .user_multi_definition_params_field_configuration import UserMultiDefinitionParamsFieldConfiguration


class UserMultiDefinitionParams(UniversalBaseModel):
    field_configuration: UserMultiDefinitionParamsFieldConfiguration
    field_type: typing.Literal["user_multi"] = "user_multi"
    name: str = pydantic.Field()
    """
    Field definition name
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel
from .anchor import Anchor
from .comment_attachment import CommentAttachment
from .link import Link
from .mention import Mention
from .text_review_annotation import TextReviewAnnotation
from .time_stamp import TimeStamp


class Comment(UniversalBaseModel):
    anchor: typing.Optional[Anchor] = pydantic.Field(default=None)
    """
    Comment anchor location. {(x, y) | 0 <= x <= 1, 0 <= y <= 1}.
    """

    annotation: typing.Optional[str] = None
    attachments: typing.Optional[typing.List[CommentAttachment]] = pydantic.Field(default=None)
    """
    Attachments associated with this comment (0-6 attachments allowed)
    """

    completed_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Completion timestamp
    """

    completer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of user who marked the comment as completed
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Creation timestamp
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    Duration of comment in frames. Requires presence of timestamp parameter
    """

    file_id: str = pydantic.Field()
    """
    File ID
    """

    id: str = pydantic.Field()
    """
    Comment ID
    """

    links: typing.List[Link] = pydantic.Field()
    """
    Links
    """

    mentions: typing.List[Mention] = pydantic.Field()
    """
    Mentions
    """

    page: typing.Optional[int] = pydantic.Field(default=None)
    """
    Document page
    """

    text: str = pydantic.Field()
    """
    Comment text
    """

    text_edited_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Text edited timestamp
    """

    text_review_annotation: typing.Optional[TextReviewAnnotation] = pydantic.Field(default=None)
    """
    Text review annotation details for comments left on text documents.
    """

    timestamp: TimeStamp
    updated_at: dt.datetime = pydantic.Field()
    """
    Update timestamp
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel
from .collection_aggregation_mode import CollectionAggregationMode
from .project import Project
from .share import Share
from .user import User


class Collection(UniversalBaseModel):
    aggregation_mode: CollectionAggregationMode
    auto_generated: bool
    created_at: dt.datetime = pydantic.Field()
    """
    Creation timestamp
    """

    creator: typing.Optional[User] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Collection description
    """

    id: str = pydantic.Field()
    """
    Collection ID
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Collection name
    """

    private: bool
    project: typing.Optional[Project] = None
    project_id: str = pydantic.Field()
    """
    Project ID
    """

    root_folder_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Root folder ID
    """

    shares: typing.Optional[typing.List[Share]] = pydantic.Field(default=None)
    """
    Shares
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Update timestamp
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import httpx
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import FrameioExperimentalEnvironment

if typing.TYPE_CHECKING:
    from .account_permissions.client import AccountPermissionsClient, AsyncAccountPermissionsClient
    from .accounts.client import AccountsClient, AsyncAccountsClient
    from .collections.client import AsyncCollectionsClient, CollectionsClient
    from .comments.client import AsyncCommentsClient, CommentsClient
    from .custom_actions.client import AsyncCustomActionsClient, CustomActionsClient
    from .files.client import AsyncFilesClient, FilesClient
    from .folder_permissions.client import AsyncFolderPermissionsClient, FolderPermissionsClient
    from .folders.client import AsyncFoldersClient, FoldersClient
    from .groups.client import AsyncGroupsClient, GroupsClient
    from .metadata.client import AsyncMetadataClient, MetadataClient
    from .metadata_fields.client import AsyncMetadataFieldsClient, MetadataFieldsClient
    from .project_permissions.client import AsyncProjectPermissionsClient, ProjectPermissionsClient
    from .projects.client import AsyncProjectsClient, ProjectsClient
    from .shares.client import AsyncSharesClient, SharesClient
    from .version_stacks.client import AsyncVersionStacksClient, VersionStacksClient
    from .webhooks.client import AsyncWebhooksClient, WebhooksClient
    from .workspace_permissions.client import AsyncWorkspacePermissionsClient, WorkspacePermissionsClient
    from .workspaces.client import AsyncWorkspacesClient, WorkspacesClient


class FrameioExperimental:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : FrameioExperimentalEnvironment
        The environment to use for requests from the client. from .environment import FrameioExperimentalEnvironment



        Defaults to FrameioExperimentalEnvironment.DEFAULT



    api_version : typing.Optional[typing.Literal["experimental"]]
    token : typing.Union[str, typing.Callable[[], str]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from frameio_experimental import FrameioExperimental

    client = FrameioExperimental(
        api_version="YOUR_API_VERSION",
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: FrameioExperimentalEnvironment = FrameioExperimentalEnvironment.DEFAULT,
        api_version: typing.Optional[typing.Literal["experimental"]] = None,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_version=api_version,
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._custom_actions: typing.Optional[CustomActionsClient] = None
        self._metadata_fields: typing.Optional[MetadataFieldsClient] = None
        self._project_permissions: typing.Optional[ProjectPermissionsClient] = None
        self._folders: typing.Optional[FoldersClient] = None
        self._version_stacks: typing.Optional[VersionStacksClient] = None
        self._account_permissions: typing.Optional[AccountPermissionsClient] = None
        self._comments: typing.Optional[CommentsClient] = None
        self._accounts: typing.Optional[AccountsClient] = None
        self._webhooks: typing.Optional[WebhooksClient] = None
        self._workspace_permissions: typing.Optional[WorkspacePermissionsClient] = None
        self._groups: typing.Optional[GroupsClient] = None
        self._projects: typing.Optional[ProjectsClient] = None
        self._metadata: typing.Optional[MetadataClient] = None
        self._files: typing.Optional[FilesClient] = None
        self._shares: typing.Optional[SharesClient] = None
        self._folder_permissions: typing.Optional[FolderPermissionsClient] = None
        self._workspaces: typing.Optional[WorkspacesClient] = None
        self._collections: typing.Optional[CollectionsClient] = None

    @property
    def custom_actions(self):
        if self._custom_actions is None:
            from .custom_actions.client import CustomActionsClient  # noqa: E402

            self._custom_actions = CustomActionsClient(client_wrapper=self._client_wrapper)
        return self._custom_actions

    @property
    def metadata_fields(self):
        if self._metadata_fields is None:
            from .metadata_fields.client import MetadataFieldsClient  # noqa: E402

            self._metadata_fields = MetadataFieldsClient(client_wrapper=self._client_wrapper)
        return self._metadata_fields

    @property
    def project_permissions(self):
        if self._project_permissions is None:
            from .project_permissions.client import ProjectPermissionsClient  # noqa: E402

            self._project_permissions = ProjectPermissionsClient(client_wrapper=self._client_wrapper)
        return self._project_permissions

    @property
    def folders(self):
        if self._folders is None:
            from .folders.client import FoldersClient  # noqa: E402

            self._folders = FoldersClient(client_wrapper=self._client_wrapper)
        return self._folders

    @property
    def version_stacks(self):
        if self._version_stacks is None:
            from .version_stacks.client import VersionStacksClient  # noqa: E402

            self._version_stacks = VersionStacksClient(client_wrapper=self._client_wrapper)
        return self._version_stacks

    @property
    def account_permissions(self):
        if self._account_permissions is None:
            from .account_permissions.client import AccountPermissionsClient  # noqa: E402

            self._account_permissions = AccountPermissionsClient(client_wrapper=self._client_wrapper)
        return self._account_permissions

    @property
    def comments(self):
        if self._comments is None:
            from .comments.client import CommentsClient  # noqa: E402

            self._comments = CommentsClient(client_wrapper=self._client_wrapper)
        return self._comments

    @property
    def accounts(self):
        if self._accounts is None:
            from .accounts.client import AccountsClient  # noqa: E402

            self._accounts = AccountsClient(client_wrapper=self._client_wrapper)
        return self._accounts

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import WebhooksClient  # noqa: E402

            self._webhooks = WebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def workspace_permissions(self):
        if self._workspace_permissions is None:
            from .workspace_permissions.client import WorkspacePermissionsClient  # noqa: E402

            self._workspace_permissions = WorkspacePermissionsClient(client_wrapper=self._client_wrapper)
        return self._workspace_permissions

    @property
    def groups(self):
        if self._groups is None:
            from .groups.client import GroupsClient  # noqa: E402

            self._groups = GroupsClient(client_wrapper=self._client_wrapper)
        return self._groups

    @property
    def projects(self):
        if self._projects is None:
            from .projects.client import ProjectsClient  # noqa: E402

            self._projects = ProjectsClient(client_wrapper=self._client_wrapper)
        return self._projects

    @property
    def metadata(self):
        if self._metadata is None:
            from .metadata.client import MetadataClient  # noqa: E402

            self._metadata = MetadataClient(client_wrapper=self._client_wrapper)
        return self._metadata

    @property
    def files(self):
        if self._files is None:
            from .files.client import FilesClient  # noqa: E402

            self._files = FilesClient(client_wrapper=self._client_wrapper)
        return self._files

    @property
    def shares(self):
        if self._shares is None:
            from .shares.client import SharesClient  # noqa: E402

            self._shares = SharesClient(client_wrapper=self._client_wrapper)
        return self._shares

    @property
    def folder_permissions(self):
        if self._folder_permissions is None:
            from .folder_permissions.client import FolderPermissionsClient  # noqa: E402

            self._folder_permissions = FolderPermissionsClient(client_wrapper=self._client_wrapper)
        return self._folder_permissions

    @property
    def workspaces(self):
        if self._workspaces is None:
            from .workspaces.client import WorkspacesClient  # noqa: E402

            self._workspaces = WorkspacesClient(client_wrapper=self._client_wrapper)
        return self._workspaces

    @property
    def collections(self):
        if self._collections is None:
            from .collections.client import CollectionsClient  # noqa: E402

            self._collections = CollectionsClient(client_wrapper=self._client_wrapper)
        return self._collections


class AsyncFrameioExperimental:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : FrameioExperimentalEnvironment
        The environment to use for requests from the client. from .environment import FrameioExperimentalEnvironment



        Defaults to FrameioExperimentalEnvironment.DEFAULT



    api_version : typing.Optional[typing.Literal["experimental"]]
    token : typing.Union[str, typing.Callable[[], str]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from frameio_experimental import AsyncFrameioExperimental

    client = AsyncFrameioExperimental(
        api_version="YOUR_API_VERSION",
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: FrameioExperimentalEnvironment = FrameioExperimentalEnvironment.DEFAULT,
        api_version: typing.Optional[typing.Literal["experimental"]] = None,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_version=api_version,
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._custom_actions: typing.Optional[AsyncCustomActionsClient] = None
        self._metadata_fields: typing.Optional[AsyncMetadataFieldsClient] = None
        self._project_permissions: typing.Optional[AsyncProjectPermissionsClient] = None
        self._folders: typing.Optional[AsyncFoldersClient] = None
        self._version_stacks: typing.Optional[AsyncVersionStacksClient] = None
        self._account_permissions: typing.Optional[AsyncAccountPermissionsClient] = None
        self._comments: typing.Optional[AsyncCommentsClient] = None
        self._accounts: typing.Optional[AsyncAccountsClient] = None
        self._webhooks: typing.Optional[AsyncWebhooksClient] = None
        self._workspace_permissions: typing.Optional[AsyncWorkspacePermissionsClient] = None
        self._groups: typing.Optional[AsyncGroupsClient] = None
        self._projects: typing.Optional[AsyncProjectsClient] = None
        self._metadata: typing.Optional[AsyncMetadataClient] = None
        self._files: typing.Optional[AsyncFilesClient] = None
        self._shares: typing.Optional[AsyncSharesClient] = None
        self._folder_permissions: typing.Optional[AsyncFolderPermissionsClient] = None
        self._workspaces: typing.Optional[AsyncWorkspacesClient] = None
        self._collections: typing.Optional[AsyncCollectionsClient] = None

    @property
    def custom_actions(self):
        if self._custom_actions is None:
            from .custom_actions.client import AsyncCustomActionsClient  # noqa: E402

            self._custom_actions = AsyncCustomActionsClient(client_wrapper=self._client_wrapper)
        return self._custom_actions

    @property
    def metadata_fields(self):
        if self._metadata_fields is None:
            from .metadata_fields.client import AsyncMetadataFieldsClient  # noqa: E402

            self._metadata_fields = AsyncMetadataFieldsClient(client_wrapper=self._client_wrapper)
        return self._metadata_fields

    @property
    def project_permissions(self):
        if self._project_permissions is None:
            from .project_permissions.client import AsyncProjectPermissionsClient  # noqa: E402

            self._project_permissions = AsyncProjectPermissionsClient(client_wrapper=self._client_wrapper)
        return self._project_permissions

    @property
    def folders(self):
        if self._folders is None:
            from .folders.client import AsyncFoldersClient  # noqa: E402

            self._folders = AsyncFoldersClient(client_wrapper=self._client_wrapper)
        return self._folders

    @property
    def version_stacks(self):
        if self._version_stacks is None:
            from .version_stacks.client import AsyncVersionStacksClient  # noqa: E402

            self._version_stacks = AsyncVersionStacksClient(client_wrapper=self._client_wrapper)
        return self._version_stacks

    @property
    def account_permissions(self):
        if self._account_permissions is None:
            from .account_permissions.client import AsyncAccountPermissionsClient  # noqa: E402

            self._account_permissions = AsyncAccountPermissionsClient(client_wrapper=self._client_wrapper)
        return self._account_permissions

    @property
    def comments(self):
        if self._comments is None:
            from .comments.client import AsyncCommentsClient  # noqa: E402

            self._comments = AsyncCommentsClient(client_wrapper=self._client_wrapper)
        return self._comments

    @property
    def accounts(self):
        if self._accounts is None:
            from .accounts.client import AsyncAccountsClient  # noqa: E402

            self._accounts = AsyncAccountsClient(client_wrapper=self._client_wrapper)
        return self._accounts

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import AsyncWebhooksClient  # noqa: E402

            self._webhooks = AsyncWebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def workspace_permissions(self):
        if self._workspace_permissions is None:
            from .workspace_permissions.client import AsyncWorkspacePermissionsClient  # noqa: E402

            self._workspace_permissions = AsyncWorkspacePermissionsClient(client_wrapper=self._client_wrapper)
        return self._workspace_permissions

    @property
    def groups(self):
        if self._groups is None:
            from .groups.client import AsyncGroupsClient  # noqa: E402

            self._groups = AsyncGroupsClient(client_wrapper=self._client_wrapper)
        return self._groups

    @property
    def projects(self):
        if self._projects is None:
            from .projects.client import AsyncProjectsClient  # noqa: E402

            self._projects = AsyncProjectsClient(client_wrapper=self._client_wrapper)
        return self._projects

    @property
    def metadata(self):
        if self._metadata is None:
            from .metadata.client import AsyncMetadataClient  # noqa: E402

            self._metadata = AsyncMetadataClient(client_wrapper=self._client_wrapper)
        return self._metadata

    @property
    def files(self):
        if self._files is None:
            from .files.client import AsyncFilesClient  # noqa: E402

            self._files = AsyncFilesClient(client_wrapper=self._client_wrapper)
        return self._files

    @property
    def shares(self):
        if self._shares is None:
            from .shares.client import AsyncSharesClient  # noqa: E402

            self._shares = AsyncSharesClient(client_wrapper=self._client_wrapper)
        return self._shares

    @property
    def folder_permissions(self):
        if self._folder_permissions is None:
            from .folder_permissions.client import AsyncFolderPermissionsClient  # noqa: E402

            self._folder_permissions = AsyncFolderPermissionsClient(client_wrapper=self._client_wrapper)
        return self._folder_permissions

    @property
    def workspaces(self):
        if self._workspaces is None:
            from .workspaces.client import AsyncWorkspacesClient  # noqa: E402

            self._workspaces = AsyncWorkspacesClient(client_wrapper=self._client_wrapper)
        return self._workspaces

    @property
    def collections(self):
        if self._collections is None:
            from .collections.client import AsyncCollectionsClient  # noqa: E402

            self._collections = AsyncCollectionsClient(client_wrapper=self._client_wrapper)
        return self._collections


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: FrameioExperimentalEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import UniversalBaseModel


class FileImportParamsData(UniversalBaseModel):
    key: str = pydantic.Field()
    """
    File key/path within the filespace
    """

    name: str = pydantic.Field()
    """
    File name
    """

    storage_location: str = pydantic.Field()
    """
    Storage location with account UUID as the value or prefix. Applies to accounts with Storage Connect
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

from .date_definition import DateDefinition
from .long_text_definition import LongTextDefinition
from .number_definition import NumberDefinition
from .rating_definition import RatingDefinition
from .select_definition import SelectDefinition
from .select_multi_definition import SelectMultiDefinition
from .text_definition import TextDefinition
from .toggle_definition import ToggleDefinition
from .user_multi_definition import UserMultiDefinition
from .user_single_definition import UserSingleDefinition

FieldDefinition = typing.Union[
    DateDefinition,
    LongTextDefinition,
    NumberDefinition,
    RatingDefinition,
    SelectDefinition,
    SelectMultiDefinition,
    TextDefinition,
    ToggleDefinition,
    UserMultiDefinition,
    UserSingleDefinition,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel
from .update_select_definition_params_field_configuration import UpdateSelectDefinitionParamsFieldConfiguration


class UpdateSelectDefinitionParams(UniversalBaseModel):
    field_configuration: typing.Optional[UpdateSelectDefinitionParamsFieldConfiguration] = None
    field_type: typing.Literal["select"] = "select"
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Field definition name
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel
from .folder import Folder


class FolderResponse(UniversalBaseModel):
    data: Folder

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import UniversalBaseModel
from ...types.anchor import Anchor
from ...types.link_params import LinkParams
from ...types.text_review_annotation_update import TextReviewAnnotationUpdate
from ...types.time_stamp import TimeStamp


class UpdateCommentParamsData(UniversalBaseModel):
    anchor: typing.Optional[Anchor] = pydantic.Field(default=None)
    """
    Comment anchor location. {(x, y) | 0 <= x <= 1, 0 <= y <= 1}.
    """

    annotation: typing.Optional[str] = pydantic.Field(default=None)
    """
    JSON geometry for on-screen drawings. Allowed for document, image, video, or stream file types. JSON must be stringified.
    """

    completed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Comment completion status
    """

    duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    Duration of comment in frames. Requires presence of timestamp parameter
    """

    links: typing.Optional[typing.List[LinkParams]] = pydantic.Field(default=None)
    """
    Comment text links.
    """

    page: typing.Optional[int] = pydantic.Field(default=None)
    """
    Document page. Only allowed when file type is a pdf document
    """

    text: typing.Optional[str] = pydantic.Field(default=None)
    """
    Comment text
    """

    text_review_annotation: typing.Optional[TextReviewAnnotationUpdate] = None
    timestamp: typing.Optional[TimeStamp] = None

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel


class AttachmentInput(UniversalBaseModel):
    """
    Input for creating a comment attachment
    """

    file_size: int = pydantic.Field()
    """
    Attachment file size in bytes (up to 5TB)
    """

    media_type: str = pydantic.Field()
    """
    File media type
    """

    name: str = pydantic.Field()
    """
    Filename
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel


class CommentAttachment(UniversalBaseModel):
    created_at: dt.datetime = pydantic.Field()
    """
    Creation timestamp
    """

    file_size: int = pydantic.Field()
    """
    Attachment file size in bytes (up to 5TB)
    """

    id: str = pydantic.Field()
    """
    Comment attachment ID
    """

    media_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    File media type
    """

    name: str = pydantic.Field()
    """
    File name for the attachment
    """

    upload_urls: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Comment attachment upload URLs. Number of URLs returned will vary depending on the file size.
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

from .boolean_value import BooleanValue
from .date_value import DateValue
from .integer_value import IntegerValue
from .multi_select_value import MultiSelectValue
from .multi_user_value import MultiUserValue
from .number_value import NumberValue
from .select_value import SelectValue
from .single_user_value import SingleUserValue
from .text_value import TextValue

MetadataField = typing.Union[
    DateValue,
    NumberValue,
    IntegerValue,
    SelectValue,
    MultiSelectValue,
    TextValue,
    BooleanValue,
    SingleUserValue,
    MultiUserValue,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import UniversalBaseModel
from .group_source import GroupSource


class Group(UniversalBaseModel):
    created_at: dt.datetime = pydantic.Field()
    """
    Creation timestamp
    """

    creator_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of group creator
    """

    emoji: str = pydantic.Field()
    """
    Group emoji for UI display
    """

    id: str = pydantic.Field()
    """
    Group ID
    """

    member_count: int = pydantic.Field()
    """
    Number of members in group
    """

    name: str = pydantic.Field()
    """
    Group name
    """

    source: GroupSource = pydantic.Field()
    """
    Group source
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Update timestamp
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.include_total_count import IncludeTotalCount
from ..types.request_after_opaque_cursor import RequestAfterOpaqueCursor
from ..types.request_page_size import RequestPageSize
from ..types.uuid_ import Uuid
from ..types.workspace_user_roles_response import WorkspaceUserRolesResponse
from .raw_client import AsyncRawWorkspacePermissionsClient, RawWorkspacePermissionsClient
from .types.workspace_user_roles_index_request_sort import WorkspaceUserRolesIndexRequestSort


class WorkspacePermissionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWorkspacePermissionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWorkspacePermissionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWorkspacePermissionsClient
        """
        return self._raw_client

    def workspace_user_roles_index(
        self,
        account_id: Uuid,
        workspace_id: Uuid,
        *,
        include_deactivated: typing.Optional[bool] = None,
        sort: typing.Optional[WorkspaceUserRolesIndexRequestSort] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkspaceUserRolesResponse:
        """
        List user roles for a given workspace. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        workspace_id : Uuid


        include_deactivated : typing.Optional[bool]
            Supports including deactivated users in the response. Default is false.

        sort : typing.Optional[WorkspaceUserRolesIndexRequestSort]
            Sort users by query params

        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkspaceUserRolesResponse
            OK

        Examples
        --------
        from frameio_experimental import FrameioExperimental

        client = FrameioExperimental(
            api_version="YOUR_API_VERSION",
            token="YOUR_TOKEN",
        )
        client.workspace_permissions.workspace_user_roles_index(
            account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            workspace_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
            include_deactivated=True,
            sort="role_asc",
            after="<opaque_cursor>",
            page_size=10,
            include_total_count=False,
        )
        """
        _response = self._raw_client.workspace_user_roles_index(
            account_id,
            workspace_id,
            include_deactivated=include_deactivated,
            sort=sort,
            after=after,
            page_size=page_size,
            include_total_count=include_total_count,
            request_options=request_options,
        )
        return _response.data


class AsyncWorkspacePermissionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWorkspacePermissionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWorkspacePermissionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWorkspacePermissionsClient
        """
        return self._raw_client

    async def workspace_user_roles_index(
        self,
        account_id: Uuid,
        workspace_id: Uuid,
        *,
        include_deactivated: typing.Optional[bool] = None,
        sort: typing.Optional[WorkspaceUserRolesIndexRequestSort] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkspaceUserRolesResponse:
        """
        List user roles for a given workspace. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        workspace_id : Uuid


        include_deactivated : typing.Optional[bool]
            Supports including deactivated users in the response. Default is false.

        sort : typing.Optional[WorkspaceUserRolesIndexRequestSort]
            Sort users by query params

        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkspaceUserRolesResponse
            OK

        Examples
        --------
        import asyncio

        from frameio_experimental import AsyncFrameioExperimental

        client = AsyncFrameioExperimental(
            api_version="YOUR_API_VERSION",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.workspace_permissions.workspace_user_roles_index(
                account_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                workspace_id="b2702c44-c6da-4bb6-8bbd-be6e547ccf1b",
                include_deactivated=True,
                sort="role_asc",
                after="<opaque_cursor>",
                page_size=10,
                include_total_count=False,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.workspace_user_roles_index(
            account_id,
            workspace_id,
            include_deactivated=include_deactivated,
            sort=sort,
            after=after,
            page_size=page_size,
            include_total_count=include_total_count,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.accounts_response import AccountsResponse
from ..types.audit_logs_with_includes_response import AuditLogsWithIncludesResponse
from ..types.bad_request import BadRequest
from ..types.conflict import Conflict
from ..types.filters import Filters
from ..types.forbidden import Forbidden
from ..types.include_total_count import IncludeTotalCount
from ..types.not_found import NotFound
from ..types.request_after_opaque_cursor import RequestAfterOpaqueCursor
from ..types.request_page_size import RequestPageSize
from ..types.too_many_requests import TooManyRequests
from ..types.unauthorized import Unauthorized
from ..types.uuid_ import Uuid
from .types.accounts_index_request_sort import AccountsIndexRequestSort


class RawAccountsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def index(
        self,
        *,
        sort: typing.Optional[AccountsIndexRequestSort] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AccountsResponse]:
        """
        List accounts for the current user. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        sort : typing.Optional[AccountsIndexRequestSort]
            Sort accounts by query params

        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AccountsResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            "v4/accounts",
            method="GET",
            params={
                "sort": sort,
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccountsResponse,
                    parse_obj_as(
                        type_=AccountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        parse_obj_as(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        parse_obj_as(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        parse_obj_as(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        parse_obj_as(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        parse_obj_as(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        parse_obj_as(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def auditlog_index(
        self,
        account_id: Uuid,
        *,
        include: typing.Optional[typing.Literal["user"]] = None,
        filters: typing.Optional[Filters] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AuditLogsWithIncludesResponse]:
        """
        List audit logs with filtering capabilities via query params.
              <br/>
              This endpoint has been added to `Current Version` of the API.
              <br/>
              Please see https://next.developer.frame.io/platform/v4/api-reference/accounts/auditlog-index
              <br/>
              Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        include : typing.Optional[typing.Literal["user"]]


        filters : typing.Optional[Filters]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AuditLogsWithIncludesResponse]
            OK
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/audit_logs",
            method="GET",
            params={
                "include": include,
                "filters": convert_and_respect_annotation_metadata(
                    object_=filters, annotation=Filters, direction="write"
                ),
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuditLogsWithIncludesResponse,
                    parse_obj_as(
                        type_=AuditLogsWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        parse_obj_as(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        parse_obj_as(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        parse_obj_as(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        parse_obj_as(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        parse_obj_as(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        parse_obj_as(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAccountsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def index(
        self,
        *,
        sort: typing.Optional[AccountsIndexRequestSort] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AccountsResponse]:
        """
        List accounts for the current user. <br>Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        sort : typing.Optional[AccountsIndexRequestSort]
            Sort accounts by query params

        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AccountsResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v4/accounts",
            method="GET",
            params={
                "sort": sort,
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AccountsResponse,
                    parse_obj_as(
                        type_=AccountsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        parse_obj_as(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        parse_obj_as(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        parse_obj_as(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        parse_obj_as(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        parse_obj_as(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        parse_obj_as(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def auditlog_index(
        self,
        account_id: Uuid,
        *,
        include: typing.Optional[typing.Literal["user"]] = None,
        filters: typing.Optional[Filters] = None,
        after: typing.Optional[RequestAfterOpaqueCursor] = None,
        page_size: typing.Optional[RequestPageSize] = None,
        include_total_count: typing.Optional[IncludeTotalCount] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AuditLogsWithIncludesResponse]:
        """
        List audit logs with filtering capabilities via query params.
              <br/>
              This endpoint has been added to `Current Version` of the API.
              <br/>
              Please see https://next.developer.frame.io/platform/v4/api-reference/accounts/auditlog-index
              <br/>
              Rate Limits: 100 calls per 1.00 minute(s) per account_user

        Parameters
        ----------
        account_id : Uuid


        include : typing.Optional[typing.Literal["user"]]


        filters : typing.Optional[Filters]


        after : typing.Optional[RequestAfterOpaqueCursor]
            Opaque Cursor query param for requests returning paginated results.
            <br/>
            NOTE: this value is auto-generated and included as part of links from a previous response. It is not intended to be human readable.

        page_size : typing.Optional[RequestPageSize]


        include_total_count : typing.Optional[IncludeTotalCount]


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AuditLogsWithIncludesResponse]
            OK
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v4/accounts/{jsonable_encoder(account_id)}/audit_logs",
            method="GET",
            params={
                "include": include,
                "filters": convert_and_respect_annotation_metadata(
                    object_=filters, annotation=Filters, direction="write"
                ),
                "after": after,
                "page_size": page_size,
                "include_total_count": include_total_count,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AuditLogsWithIncludesResponse,
                    parse_obj_as(
                        type_=AuditLogsWithIncludesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BadRequest,
                        parse_obj_as(
                            type_=BadRequest,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Unauthorized,
                        parse_obj_as(
                            type_=Unauthorized,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Forbidden,
                        parse_obj_as(
                            type_=Forbidden,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        NotFound,
                        parse_obj_as(
                            type_=NotFound,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Conflict,
                        parse_obj_as(
                            type_=Conflict,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TooManyRequests,
                        parse_obj_as(
                            type_=TooManyRequests,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from .asset_common import AssetCommon


class Folder(AssetCommon):
    cover_file_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cover asset ID
    """

    restricted: bool = pydantic.Field()
    """
    Folder is restricted or not
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import UniversalBaseModel


class FolderUpdateParamsData(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Folder name
    """

    restricted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Folder is restricted or not.
    """

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.date_definition_params import DateDefinitionParams
from ...types.long_text_definition_params import LongTextDefinitionParams
from ...types.number_definition_params import NumberDefinitionParams
from ...types.rating_definition_params import RatingDefinitionParams
from ...types.select_definition_params import SelectDefinitionParams
from ...types.select_multi_definition_params import SelectMultiDefinitionParams
from ...types.text_definition_params import TextDefinitionParams
from ...types.toggle_definition_params import ToggleDefinitionParams
from ...types.user_multi_definition_params import UserMultiDefinitionParams
from ...types.user_single_definition_params import UserSingleDefinitionParams

CreateFieldDefinitionParamsData = typing.Union[
    DateDefinitionParams,
    LongTextDefinitionParams,
    NumberDefinitionParams,
    RatingDefinitionParams,
    SelectDefinitionParams,
    SelectMultiDefinitionParams,
    TextDefinitionParams,
    ToggleDefinitionParams,
    UserMultiDefinitionParams,
    UserSingleDefinitionParams,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from .field_value_common import FieldValueCommon
from .select_option import SelectOption


class SelectValue(FieldValueCommon):
    field_options: typing.Optional[typing.List[SelectOption]] = None
    value: typing.Optional[typing.List[SelectOption]] = None

    model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)



# This file was auto-generated by Fern from our API Definition.

import typing

WorkspacesIndexRequestSort = typing.Union[
    typing.Literal["name_asc", "created_at_asc", "name_desc", "created_at_desc"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

TextReviewAnnotationUpdateTextReviewAnnotationType = typing.Union[
    typing.Literal["highlight", "insert", "remove", "replace"], typing.Any
]
